import 'dart:io';

import 'android_assets.dart';
import 'package:path/path.dart' as path;

const _iconFileName = 'ic_launcher.png';
const _backgroundFileName = 'ic_launcher_background.png';
const _foregroundFileName = 'ic_launcher_foreground.png';

class AndroidAssetsExtractor {
  final String _launcherIconsDirPath;
  late final _androidDirPath = path.join(_launcherIconsDirPath, 'android');

  AndroidAssetsExtractor(this._launcherIconsDirPath);

  /// extracts assets from from assets/launcher_icons/android/*
  /// provided the class was created with
  /// AndroidAssetsExtractor('assets/launcher_icons')
  Future<AndroidAssets?> findAll() async {
    final androidDirectory = Directory(_androidDirPath);

    if (androidDirectory.existsSync()) {
      return _extractAssetGroup(androidDirectory);
    } else {
      return null;
    }
  }

  Future<AndroidAssets> _extractAssetGroup(
    Directory dir,
  ) async {
    final iconPath = path.join(dir.path, _iconFileName);
    final foregroundPath = path.join(dir.path, _foregroundFileName);
    final backgroundPath = path.join(dir.path, _backgroundFileName);
    final iconFile = File(iconPath);
    final foregroundFile = File(foregroundPath);
    final backgroundFile = File(backgroundPath);

    return AndroidAssets(
      icon: iconFile,
      background: backgroundFile,
      foreground: foregroundFile,
    );
  }
}
