import 'dart:io';

import 'package:image/image.dart';
import 'package:launcher_icons_generator/file_writters/file_writter.dart';

import '../../utils/image_decoder.dart';
import 'ios_icon_template.dart';
import 'package:path/path.dart' as path;

class IosFileWritter extends FileWritter {
  static const iconTemplates = [
    IosIconTemplate(name: 'Icon-App-20x20@1x.png', size: 20),
    IosIconTemplate(name: 'Icon-App-20x20@2x.png', size: 40),
    IosIconTemplate(name: 'Icon-App-20x20@3x.png', size: 60),
    IosIconTemplate(name: 'Icon-App-29x29@1x.png', size: 29),
    IosIconTemplate(name: 'Icon-App-29x29@2x.png', size: 58),
    IosIconTemplate(name: 'Icon-App-29x29@3x.png', size: 87),
    IosIconTemplate(name: 'Icon-App-40x40@1x.png', size: 40),
    IosIconTemplate(name: 'Icon-App-40x40@2x.png', size: 80),
    IosIconTemplate(name: 'Icon-App-40x40@3x.png', size: 120),
    IosIconTemplate(name: 'Icon-App-60x60@2x.png', size: 120),
    IosIconTemplate(name: 'Icon-App-60x60@3x.png', size: 180),
    IosIconTemplate(name: 'Icon-App-76x76@1x.png', size: 76),
    IosIconTemplate(name: 'Icon-App-76x76@2x.png', size: 152),
    IosIconTemplate(name: 'Icon-App-83.5x83.5@2x.png', size: 167),
    IosIconTemplate(name: 'Icon-App-1024x1024@1x.png', size: 1024),
  ];

  final _decoder = ImageDecoder();

  Future<void> generate({
    required File iconFile,
  }) async {
    if (!iconFile.existsSync()) {
      throw 'icon file ${iconFile.path} not found';
    }
    final icon = await _decoder.decodeImageFile(iconFile);
    await _writeImageFromTemplates(icon, iconTemplates);
  }

  Future<void> _writeImageFromTemplates(
    Image image,
    List<IosIconTemplate> templates,
  ) async {
    final allProgress = <Future>[];
    for (final template in templates) {
      final size = template.size;
      final path = template.buildPath();
      final resizedImage = _decoder.createResizedImage(size, image);
      final progress = writeIconFile(path, resizedImage);
      allProgress.add(progress);
    }
    await Future.wait<void>(allProgress);
  }
}
