import 'dart:io';

import 'package:image/image.dart';
import 'package:launcher_icons_generator/file_writters/file_writter.dart';
import 'package:launcher_icons_generator/file_writters/web/web_icon_template.dart';
import 'package:launcher_icons_generator/file_writters/web/web_paths.dart';
import 'package:launcher_icons_generator/utils/image_decoder.dart';
import 'package:path/path.dart' as path;

class WebFileWritter extends FileWritter {
  final _decoder = ImageDecoder();
  static const faviconSize = 32;

  static const templates = [
    WebIconTemplate(size: 192, fileName: 'Icon-192.png'),
    WebIconTemplate(size: 512, fileName: 'Icon-512.png'),
    WebIconTemplate(size: 192, fileName: 'Icon-maskable-192.png'),
    WebIconTemplate(size: 512, fileName: 'Icon-maskable-512.png'),
  ];

  Future<void> generate({required File icon}) async {
    if (!icon.existsSync()) {
      throw 'icon file ${icon.path} not found';
    }
    final image = await _decoder.decodeImageFile(icon);
    await _generateFavicon(image);
    await _generateIcons(image);
  }

  Future<void> _generateFavicon(Image image) async {
    final resizedImage = _decoder.createResizedImage(faviconSize, image);
    final faviconPath = path.join(webPath, faviconFileName);
    await writeIconFile(faviconPath, resizedImage);
  }

  Future<void> _generateIcons(Image image) async {
    for (final template in templates) {
      final resizedImage = _decoder.createResizedImage(template.size, image);
      final iconPath = path.join(webIconsPath, template.fileName);
      await writeIconFile(iconPath, resizedImage);
    }
  }
}
