import 'dart:io';

import 'package:image/image.dart';
import 'package:test/test.dart';
import 'package:path/path.dart' as path;

import 'expectations.dart';
import 'setup_utils.dart';

void main() async {
  setUpAll(() async {
    await copyComplexAssetsIntoApp();
    await runCli();
  });

  tearDownAll(() async {
    await removeFlutterApp();
  });

  group('android', () {
    test('should generate android icons', () async {
      for (final entry in androidExpectedIconsFolders.entries) {
        final iconPath = path.join(
          androidIconDir,
          entry.key,
          androidIconName,
        );
        final file = File(iconPath);
        final image = decodePng(file.readAsBytesSync());
        expect(file.existsSync(), isTrue);
        expect(image?.width, equals(entry.value));
        expect(image?.height, equals(entry.value));
      }
    });

    test('should generate android adaptive icons', () {
      for (final entry in androidExpectedAdaptiveIconsFolders.entries) {
        final foregroundIconPath = path.join(
          androidIconDir,
          entry.key,
          androidForegroundName,
        );
        final backgroundIconPath = path.join(
          androidIconDir,
          entry.key,
          androidBackgroundName,
        );
        final foregroundFile = File(foregroundIconPath);
        final backgroundFile = File(backgroundIconPath);
        final foregroundImage = decodePng(foregroundFile.readAsBytesSync());
        final backgroundImage = decodePng(backgroundFile.readAsBytesSync());
        expect(foregroundFile.existsSync(), isTrue);
        expect(backgroundFile.existsSync(), isTrue);
        expect(foregroundImage?.width, equals(entry.value));
        expect(foregroundImage?.height, equals(entry.value));
        expect(backgroundImage?.width, equals(entry.value));
        expect(backgroundImage?.height, equals(entry.value));
      }
    });

    test('should generate adaptive icons xml', () {
      final xmlPath = path.join(
        androidIconDir,
        androidAdaptiveIconXmlFolder,
        androidAdaptiveIconName,
      );
      final file = File(xmlPath);
      final content = file.readAsStringSync().replaceAll(RegExp(r'\s*'), '');
      final expectedContent =
          androidAdaptiveIconXmlContent.replaceAll(RegExp(r'\s*'), '');
      expect(file.existsSync(), isTrue);
      expect(content, equals(expectedContent));
    });
  });

  test('should generate web icons', () {
    final faviconFile = File(path.join(webPath, webFavicon));
    expect(faviconFile.existsSync(), isTrue);
    final faviconImage = decodePng(faviconFile.readAsBytesSync());
    expect(faviconImage?.width, equals(webFaviconSize));
    expect(faviconImage?.height, equals(webFaviconSize));
    for (final iconEntry in webIcons.entries) {
      final file = File(path.join(webPath, 'icons', iconEntry.key));
      expect(file.existsSync(), isTrue);
      final image = decodePng(file.readAsBytesSync());
      expect(image?.width, equals(iconEntry.value));
      expect(image?.height, equals(iconEntry.value));
    }
  });

  test('should generate ios icons', () {
    for (final iconEntry in iosIcons.entries) {
      final file = File(path.join(iosIconDir, iconEntry.key));
      expect(file.existsSync(), isTrue);
      final image = decodePng(file.readAsBytesSync());
      expect(image?.width, equals(iconEntry.value));
      expect(image?.height, equals(iconEntry.value));
    }
  });
}
