import 'dart:io';

import 'package:image/image.dart';
import 'package:test/test.dart';
import 'package:path/path.dart' as path;

import 'expectations.dart';
import 'setup_utils.dart';

void main() async {
  setUpAll(() async {
    await copySimpleAssetsIntoApp();
    await runCli();
  });

  tearDownAll(() async {
    await removeFlutterApp();
  });

  test('should generate android icons', () async {
    for (final entry in androidExpectedIconsFolders.entries) {
      final iconPath = path.join(
        androidIconDir,
        entry.key,
        androidIconName,
      );
      final file = File(iconPath);
      final image = decodePng(file.readAsBytesSync());
      expect(file.existsSync(), isTrue);
      expect(image?.width, equals(entry.value));
      expect(image?.height, equals(entry.value));
    }
  });

  test('should generate web icons', () {
    final faviconFile = File(path.join(webPath, webFavicon));
    expect(faviconFile.existsSync(), isTrue);
    final faviconImage = decodePng(faviconFile.readAsBytesSync());
    expect(faviconImage?.width, equals(webFaviconSize));
    expect(faviconImage?.height, equals(webFaviconSize));
    for (final iconEntry in webIcons.entries) {
      final file = File(path.join(webIconDir, iconEntry.key));
      expect(file.existsSync(), isTrue);
      final image = decodePng(file.readAsBytesSync());
      expect(image?.width, equals(iconEntry.value));
      expect(image?.height, equals(iconEntry.value));
    }
  });

  test('should generate ios icons', () {
    for (final iconEntry in iosIcons.entries) {
      final file = File(path.join(iosIconDir, iconEntry.key));
      expect(file.existsSync(), isTrue);
      final image = decodePng(file.readAsBytesSync());
      expect(image?.width, equals(iconEntry.value));
      expect(image?.height, equals(iconEntry.value));
    }
  });
}
